package com.ywt.outpatient.taihe.rpc.domain.entity.center;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;

import javax.transaction.Transactional;
import java.util.List;

/**
 * RegisteredOrder Repository
 *
 * @author johnson lin
 * @date 2019/4/30 5:20 PM
 */
public interface RegisteredOrderRepository extends JpaRepository<RegisteredOrder, Integer> {
    /**
     * 获取预约挂号订单
     *
     * @param orderId    订单ID
     * @param hospitalId 医院ID
     * @return {@link RegisteredOrder}
     */
    RegisteredOrder getFirstByOrderIdAndHospitalIdAndDeletedFalse(int orderId, int hospitalId);

    /**
     * 获取预约挂号订单
     *
     * @param id RegisteredOrder表主键
     * @return {@link RegisteredOrder}
     */
    RegisteredOrder getById(int id);

    /**
     * 更新订单ID、订单编码
     *
     * @param orderId 订单ID
     * @param orderNo 订单编码
     * @param id      主键
     */
    @Modifying
    @Transactional
    @Query(value = "update registered_order set order_id = ?1, order_no = ?2 where id = ?3", nativeQuery = true)
    void updateOrderIdAndOrderNoById(int orderId, String orderNo, int id);

    /**
     * 更新挂号信息
     *
     * @param hisSeqCode       挂号序号
     * @param hisRegFee        挂号费
     * @param hisAdmitRange    就诊时间段
     * @param hisAdmitAddress  就诊地点
     * @param hisOrderContent  预约单内容（内容说明：卡号^病人Rowid^病人姓名^性别^挂号序号(班别)^挂号序号^科室名称^医生姓名^预约日期^时间^候诊日期^^）
     * @param hisTransactionId his系统交易流水号
     * @param hisOrderCode     预约单号
     * @param registeredStatus 订单状态
     * @param id               主键
     */
    @Modifying
    @Transactional
    @Query(value =
            "update registered_order " +
                    "set his_seq_code = ?1" +
                    ", his_reg_fee = ?2" +
                    ", his_admit_range = ?3" +
                    ", his_admit_address = ?4" +
                    ", his_order_content = ?5" +
                    ", his_transaction_id = ?6" +
                    ", his_order_code = ?7" +
                    ", registered_status = ?8 " +
                    ", his_clinic_code = ?10 " +
                    "where id = ?9"
            , nativeQuery = true)
    void updateRegisterInfoById(
            String hisSeqCode, String hisRegFee, String hisAdmitRange, String hisAdmitAddress,
            String hisOrderContent, String hisTransactionId, String hisOrderCode, int registeredStatus, int id,
            String hisClinicCode);

    /**
     * 更新 registered_status 字段
     *
     * @param registeredStatus 预约挂号状态
     * @param id               主键
     */
    @Modifying
    @Transactional
    @Query(value = "update registered_order set registered_status = registered_status | ?1 where id = ?2", nativeQuery = true)
    void updateRegisteredStatus(int registeredStatus, int id);

    /**
     * 获取用户候诊列表记录（太和医院）
     *
     * @param userId 用户ID
     * @param date   日期，格式：yyyy-M-d
     * @return 候诊列表
     */
    @Query(value =
            "select * from registered_order " +
                    "where user_id = ?1 and deleted = 0 and hospital_id = ?3 " +
                    "and registered_date >= ?2 " +
                    "and (registered_status = 1 or registered_status = 5) " +
                    "order by create_time desc;"
            , nativeQuery = true)
    List<RegisteredOrder> listQueueOrders(int userId, String date, int hospitalId);

    @Query(value = "select * from ywt_center.registered_order where doctor_id = ? and deleted = 0 and user_id = ? and payment_status = 2 and registered_status & 1= 1 and registered_date <= ? order by registered_date desc limit 1;", nativeQuery = true)
    RegisteredOrder getLatestRegOrderUntilDate(int doctorId, int userId, String endDate);




    @Query(value = "select * from ywt_center.registered_order where doctor_id = ? and deleted = 0 and user_id = ? and payment_status = 2 and registered_status & 1= 1 and registered_date <= ? and hospital_id = ? order by registered_date desc limit 1;", nativeQuery = true)
    RegisteredOrder getLatestRegOrderUntilDate(int doctorId, int userId, String endDate, int hospitalId);
    

    @Query(value = "select * from ywt_center.registered_order where user_id = ? and hospital_id = ? and patient_name = ? and deleted = 0 and registered_status = 1 and registered_date = ? order by registered_date desc;", nativeQuery = true)
    List<RegisteredOrder> getRegOrderList(int userId, int hospitalId, String name, String date);

    RegisteredOrder getFirstByRefundNoAndDeletedFalse(String orderNo);
}
